@@ -1548,7 +1548,7 @@ let init_relevant env pi rs =
   let push e r = r := e :: !r in
   let do1 p ax =
     let wanted = wanted_ax p in
-    if wanted || (ax.ax_visibility = `Visible && not (unwanted_ax p)) then begin
+    if wanted || (ax.ax_smt && not (unwanted_ax p)) then begin
       Frequency.add fr ax.ax_spec;
       let used = Frequency.f_ops unwanted_ops ax.ax_spec in
       let paxu = (p,ax), used in
@@ -1639,7 +1639,7 @@ let init hyps concl =
 let select env pi hyps concl execute_task =
   if pi.P.pr_all then
     let init_select p ax =
-      ax.ax_visibility = `Visible && not (P.Hints.mem p pi.P.pr_unwanted) in
+      ax.ax_smt && not (P.Hints.mem p pi.P.pr_unwanted) in
     (execute_task (EcEnv.Ax.all ~check:init_select env) = Some true)
   else
     let rs = Frequency.f_ops_goal unwanted_ops hyps.h_local concl in
@@ -1724,7 +1724,15 @@ let check ?notify (pi : P.prover_infos) (hyps : LDecl.hyps) (concl : form) =
           Format.eprintf "dumping in %s" filename;
       let filename = Printf.sprintf "%.4d-%s" tkid filename in
       out_task filename task));
-    let (tp, res) = EcUtils.timed (P.execute_task ?notify pi) task in
+    let (tp, res) = EcUtils.timed (fun task ->
+        if Sys.ocaml_release.major = 5 then begin
+          let control = Gc.get () in
+          Gc.set { control with space_overhead = min 20 control.space_overhead; };
+          EcUtils.try_finally
+            (fun () -> P.execute_task ?notify pi task)
+            (fun () -> Gc.set control)
+        end else P.execute_task ?notify pi task
+      ) task in
 
     if 1 <= pi.P.pr_verbose then
       notify |> oiter (fun notify -> notify `Warning (lazy (
