@@ -25,16 +25,21 @@ type 'a ovrenv = {
 
 and 'a ovrhooks = {
   henv      : 'a -> EcSection.scenv;
-  hadd_item : 'a -> EcTheory.import -> EcTheory.theory_item_r -> 'a;
+  hadd_item : 'a -> import:bool -> EcTheory.theory_item_r -> 'a;
   hthenter  : 'a -> thmode -> symbol -> EcTypes.is_local -> 'a;
-  hthexit   : 'a -> [`Full | `ClearOnly | `No] -> 'a;
+  hthexit   : 'a -> import:bool -> [`Full | `ClearOnly | `No] -> 'a;
   herr      : 'b . ?loc:EcLocation.t -> string -> 'b;
 }
 
 (* -------------------------------------------------------------------- *)
 val replay : 'a ovrhooks
-  -> abstract:bool -> override_locality:EcTypes.is_local option -> incl:bool
-  -> clears:Sp.t -> renames:(renaming list)
-  -> opath:path -> npath:path -> evclone
-  -> 'a -> symbol * theory_item list * EcTypes.is_local
+  -> abstract:bool
+  -> override_locality:EcTypes.is_local option
+  -> incl:bool
+  -> clears:Sp.t
+  -> renames:(renaming list)
+  -> opath:path
+  -> npath:path
+  -> evclone
+  -> 'a -> symbol * bool * theory_item list * EcTypes.is_local
   ->  axclone list * 'a
